rm(list=ls())
graphics.off()

d <- read.csv('simple.txt')
d$class <- as.factor(d$class)

plot(d$x1,d$x2,col=d$class)

library(rpart)
library(rpart.plot)

stump <- rpart(class ~ .,data=d,control=rpart.control(maxdepth=1))
rpart.plot(stump)
plot(d$x1,d$x2,col=predict(stump,newdata=d,type='class'))

tree <- rpart(class ~ .,data=d)
rpart.plot(tree)
plot(d$x1,d$x2,col=predict(tree,newdata=d,type='class'))

# bagging

nstump <- 100

stumps <- list()
for(i in 1:nstump){
  ind <- sample(1:nrow(d),nrow(d),replace=T)
  stumps[[i]] <- rpart(class ~ .,
                       data=d[ind,],
                       control=rpart.control(maxdepth=1))
}

pred <- lapply(stumps,function(x){as.character(predict(x,newdata=d,type='class'))})
pred <- do.call('cbind',pred)
pred <- apply(pred,1,function(x) {tab <- table(x);return(names(tab)[which.max(tab)])})
pred <- as.factor(pred)

plot(d$x1,d$x2,col=pred)

# adaboost

nstump <- 100

stumpProbs <- rep(1/nrow(d),times=nrow(d))

stumps <- list()
for(i in 1:nstump){
  ind <- sample(1:nrow(d),nrow(d),prob=stumpProbs,replace=T)
  stumps[[i]] <- rpart(class ~ .,
                       data=d[ind,],
                       control=rpart.control(maxdepth=1))
  err <- sum(stumpProbs*(predict(stumps[[i]],newdata=d,type='class') != d$class))/sum(stumpProbs)
  
  pred <- lapply(stumps,function(x){as.character(predict(x,newdata=d,type='class'))})
  pred <- do.call('cbind',pred)
  pred <- apply(pred,1,function(x) {tab <- table(x);return(names(tab)[which.max(tab)])})
  pred <- as.factor(pred)
  
  acc <- sum(pred == d$class)/nrow(d)
  
  plot(d$x1,d$x2,col=pred,main=paste('Iteration: ',i))
  
  print(paste('Iteration: ',i," Accuracy: ",round(acc,digits=3)," Weighted error: ",round(err,digits=3)))
  
  if(err > 0.5){
    break
  }else{
    alpha <- (1/2)*log((1 - err)/err)
    stumpProbs <- stumpProbs*exp(alpha*(predict(stumps[[i]],newdata=d,type='class') != d$class))
    stumpProbs <- stumpProbs/sum(stumpProbs)
  }
}

pred <- lapply(stumps,function(x){as.character(predict(x,newdata=d,type='class'))})
pred <- do.call('cbind',pred)
pred <- apply(pred,1,function(x) {tab <- table(x);return(names(tab)[which.max(tab)])})
pred <- as.factor(pred)

plot(d$x1,d$x2,col=pred,main='Final')








